package com.xy.job;

import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xy.collections.list.JArrayList;
import com.xy.collections.map.JMap;
import com.xy.dto.MercAccountDto;
import com.xy.entity.DeviceAlgorithmCharging;
import com.xy.entity.DeviceInfo;
import com.xy.service.DeviceAlgorithmChargingServiceImpl;
import com.xy.service.DeviceInfoServiceImpl;
import com.xy.service.MercAccountService;
import com.xy.utils.DataTime;
import com.xy.utils.Emptys;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * 设备算法费job
 */
@Slf4j
@Component
@AllArgsConstructor
public class DeviceAlgorithmChargingJob {

    private MercAccountService mercAccountService;

    private DeviceInfoServiceImpl deviceInfoService;

    private DeviceAlgorithmChargingServiceImpl deviceAlgorithmChargingService;

    /**
     * 算法费设备停机
     *
     * @return
     */
    @XxlJob("deviceAlgorithmChargingHalt")
    public ReturnT<String> deviceAlgorithmChargingHalt() {
        //查询余额<=0商户
        List<MercAccountDto.Vo> mercAccounts = mercAccountService.list(new MercAccountDto.SelectList().setEndBalance(0)).getData();
        if (!Emptys.check(mercAccounts)) {
            return ReturnT.SUCCESS;
        }
        mercAccounts.forEach(mercAccount -> {
            List<DeviceInfo> updateDeviceInfos = new ArrayList<>();
            //查询商户未冻结设备
            List<DeviceInfo> deviceInfos = deviceInfoService.list(new LambdaQueryWrapper<DeviceInfo>()
                    .eq(DeviceInfo::getMercId, mercAccount.getMercId())
                    .eq(DeviceInfo::getActiveState, 1)
                    .eq(DeviceInfo::getFreezeStatus, 1)
            );
            if (!Emptys.check(deviceInfos)) {
                return;
            }
            JMap<Long, DeviceInfo> deviceInfosJMaps = new JArrayList<>(deviceInfos).toMap(DeviceInfo::getDeviceId).cover();
            //分组统计设备可用算法卡包
            String algorithmDate = DateUtil.format(DateUtil.date(), DatePattern.PURE_DATE_PATTERN);
            LambdaQueryWrapper<DeviceAlgorithmCharging> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceAlgorithmCharging>()
                    .select(DeviceAlgorithmCharging::getDeviceId)
                    .gt(DeviceAlgorithmCharging::getMercId, mercAccount.getMercId())
                    .gt(DeviceAlgorithmCharging::getUnusedSize, 0)
                    .in(DeviceAlgorithmCharging::getDeviceId, deviceInfosJMaps.getKeys())
                    .and(deviceAlgorithmChargingLambdaQueryWrapper -> deviceAlgorithmChargingLambdaQueryWrapper
                            .eq(DeviceAlgorithmCharging::getTimeout, -1)
                            .or(deviceAlgorithmChargingLambdaQueryWrapper1 -> deviceAlgorithmChargingLambdaQueryWrapper1
                                    .le(DeviceAlgorithmCharging::getBeginTime, Integer.valueOf(algorithmDate))
                                    .ge(DeviceAlgorithmCharging::getTimeout, Integer.valueOf(algorithmDate))
                            )
                    )
                    .groupBy(DeviceAlgorithmCharging::getDeviceId);
            List<DeviceAlgorithmCharging> deviceAlgorithmChargings = deviceAlgorithmChargingService.list(lambdaQueryWrapper);
            JMap<Long, DeviceAlgorithmCharging> deviceAlgorithmChargingsJMaps = new JArrayList<>(deviceAlgorithmChargings).toMap(DeviceAlgorithmCharging::getDeviceId).cover();
            //判断是否过期
            LocalDateTime now = LocalDateTime.now();
            deviceInfosJMaps.forEach((deviceId, deviceInfo) -> {
                if (deviceAlgorithmChargingsJMaps.containsKey(deviceId)) {
                    return;
                }
                //冻结设备
                DeviceInfo updateDeviceInfo = new DeviceInfo()
                        .setDeviceId(deviceId)
                        .setFreezeStatus(2)
                        .setUpdateTime(now);
                updateDeviceInfos.add(updateDeviceInfo);
            });
            if (Emptys.check(updateDeviceInfos)) {
                deviceInfoService.updateBatchById(updateDeviceInfos);
            }
        });
        return ReturnT.SUCCESS;
    }
}
