package com.xy.job;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import com.xy.collections.list.JArrayList;
import com.xy.collections.map.JHashMap;
import com.xy.collections.map.JMap;
import com.xy.dto.MercMsgInfoDto;
import com.xy.entity.DeviceCharging;
import com.xy.entity.DeviceInfo;
import com.xy.entity.SysDictRedis;
import com.xy.mapper.DeviceChargingMapper;
import com.xy.service.DeviceInfoServiceImpl;
import com.xy.service.MercMsgInfoService;
import com.xy.sys.EnumMercCostMsgConfig;
import com.xy.utils.*;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * 设备管理费job
 */
@Slf4j
@Component
@AllArgsConstructor
public class DeviceChargingJob {

    private DeviceChargingMapper deviceChargingMapper;

    private MercMsgInfoService mercMsgInfoService;

    private DeviceInfoServiceImpl deviceInfoService;

    /**
     * 管理费设备停机
     *
     * @return
     */
    @XxlJob("deviceChargingHalt")
    public ReturnT<String> deviceChargingHalt() {
        LocalDateTime now = LocalDateTime.now();
        Map<String, SysDictRedis> stringSysDictRedisMap = SysDictUtils.get(EnumMercCostMsgConfig.Code.CODE.getCode());
        int day = Integer.valueOf(stringSysDictRedisMap.get(EnumMercCostMsgConfig.ARREARAGE_DAY.getCode()).getValue());
        String timeout = DataTime.getStringAround(0, 0, (~(day - 1)), 0, 0, 0);
        LambdaQueryWrapper<DeviceCharging> lambdaQueryWrapper = new LambdaQueryWrapper<DeviceCharging>()
                .select(DeviceCharging::getDeviceId)
                .le(DeviceCharging::getTimeout, timeout);
        RecursionUtils.recursion(current -> {
            //查询已过期设备管理费
            IPage<DeviceCharging> iPage = deviceChargingMapper.selectPage(Page.of(current, 200), lambdaQueryWrapper);
            List<DeviceCharging> records = iPage.getRecords();
            if (!Emptys.check(records)) {
                return false;
            }
            //查询未冻结设备
            LambdaQueryWrapper<DeviceInfo> deviceInfoLambdaQueryWrapper = new LambdaQueryWrapper<DeviceInfo>()
                    .select(DeviceInfo::getDeviceId)
                    .in(DeviceInfo::getDeviceId, new JArrayList<>(records).getProperty(DeviceCharging::getDeviceId))
                    .eq(DeviceInfo::getActiveState, 1)
                    .eq(DeviceInfo::getFreezeStatus, 1);
            List<DeviceInfo> deviceInfos = deviceInfoService.list(deviceInfoLambdaQueryWrapper);
            //冻结设备
            if (Emptys.check(deviceInfos)) {
                deviceInfos.forEach(deviceInfo -> deviceInfo
                        .setFreezeStatus(2)
                        .setUpdateTime(now)
                );
                deviceInfoService.updateBatchById(deviceInfos);
            }
            return true;
        });
        return ReturnT.SUCCESS;
    }

    /**
     * 设备管理费过期消息
     *
     * @return
     */
    @XxlJob("deviceChargingTimeout")
    public ReturnT<String> deviceChargingTimeout() {
        String now = DataTime.getSring();
        Map<String, SysDictRedis> stringSysDictRedisMap = SysDictUtils.get(EnumMercCostMsgConfig.Code.CODE.getCode());
        String tel = stringSysDictRedisMap.get(EnumMercCostMsgConfig.CUSTOM_TEL.getCode()).getValue();
        String mmiTime = DataTime.getStringAround(0, -1, 0, 0, 0, 0, now);
        //即将过期
        FunctionUtils.NoParamsNoResult timeout1 = () -> {
            int day = Integer.valueOf(stringSysDictRedisMap.get(EnumMercCostMsgConfig.DEVICE_COST_DAY.getCode()).getValue());
            String end = DataTime.getStringAround(0, 0, day, 0, 0, 0, now);
            JMap<String, Object> map = new JHashMap<String, Object>()
                    .set("mmiTime", mmiTime)
                    .set("workType", 1)
                    .set("begin", now)
                    .set("end", end);
            deviceChargingTimeout1(map, 1, 100, tel);
        };
        //欠费
        FunctionUtils.NoParamsNoResult timeout2 = () -> {
            int day = Integer.valueOf(stringSysDictRedisMap.get(EnumMercCostMsgConfig.ARREARAGE_DAY.getCode()).getValue());
            String time1 = DataTime.getStringAround(0, 0, day, 0, 0, 0, now);
            JMap<String, Object> map = new JHashMap<String, Object>()
                    .set("mmiTime", mmiTime)
                    .set("workType", 2)
                    .set("end", now);
            deviceChargingTimeout2(map, now, time1, 1, 100, tel);
        };
        timeout1.run();
        timeout2.run();
        return ReturnT.SUCCESS;
    }

    /**
     * 即将过期
     *
     * @param map     sql参数
     * @param current 当前页码
     * @param size    显示行数
     * @param tel     客服手机
     */
    private void deviceChargingTimeout1(JMap<String, Object> map, long current, long size, String tel) {
        //查询数据
        IPage<Long> longIPage = deviceChargingMapper.deviceChargingTimeout(Page.of(current, size), map);
        List<Long> mercIds = longIPage.getRecords();
        if (!Emptys.check(mercIds)) {
            return;
        }
        //发送消息
        JMap<String, Object> officialParams = new JHashMap<String, Object>()
                .set("content", String.format("尊敬的客户，您有设备管理费即将到期，为避免设备停机，请及时续费，客服电话%s", tel));
        JMap<String, Object> mxrParams = officialParams;
        MsgUtils.sendMsg(17L, mercIds)
                .official(userInfo -> new MsgUtils.MsgInfo(userInfo.getMpOpenid(), officialParams))
                .mxr(userInfo -> new MsgUtils.MsgInfo(String.valueOf(userInfo.getAuthorizeUserId()), mxrParams))
                .builder();
        //写入记录
        mercMsgInfoService.save(new MercMsgInfoDto.Save()
                .setMercId(mercIds)
                .setType(1)
                .setWorkType(1)
        );
        //下一页
        current++;
        deviceChargingTimeout1(map, current, size, tel);
    }

    /**
     * 已欠费
     *
     * @param map      sql参数
     * @param thisTime 当前时间
     * @param time1    过期后可用时间
     * @param current  当前页码
     * @param size     显示行数
     * @param tel      客服手机
     */
    private void deviceChargingTimeout2(JMap<String, Object> map, String thisTime, String time1, long current, long size, String tel) {
        //查询数据
        IPage<Long> longIPage = deviceChargingMapper.deviceChargingTimeout(Page.of(current, size), map);
        List<Long> mercIds = longIPage.getRecords();
        if (!Emptys.check(mercIds)) {
            return;
        }
        //发送消息
        JMap<String, Object> officialParams = new JHashMap<String, Object>()
                .set("content", String.format("尊敬的客户，截止%s您的%s已欠费，如您在%s前未完成续费，您的设备将停止交易，客服电话%s", thisTime, "设备管理费", time1, tel));
        JMap<String, Object> mxrParams = officialParams;
        MsgUtils.sendMsg(17L, mercIds)
                .official(userInfo -> new MsgUtils.MsgInfo(userInfo.getMpOpenid(), officialParams))
                .mxr(userInfo -> new MsgUtils.MsgInfo(String.valueOf(userInfo.getAuthorizeUserId()), mxrParams))
                .builder();
        //写入记录
        mercMsgInfoService.save(new MercMsgInfoDto.Save()
                .setMercId(mercIds)
                .setType(1)
                .setWorkType(2)
        );
        //下一页
        current++;
        deviceChargingTimeout2(map, thisTime, time1, current, size, tel);
    }

}
